{% extends "layout.html" %}

{% comment %}
Copyright (C) 2024  The Software Heritage developers
See the AUTHORS file at the top-level directory of this distribution
License: GNU Affero General Public License version 3, or any later version
See top-level LICENSE file for more information
{% endcomment %}

{% block page_title %}
  Access restricted (Error {{ error_code }})
{% endblock page_title %}

{% block navbar-content %}
  <h4>Access restricted</h4>
{% endblock navbar-content %}

{% block content %}
  {% include "includes/masking-error.html" %}
{% endblock content %}
