!function(){try{var e="undefined"!=typeof window?window:"undefined"!=typeof global?global:"undefined"!=typeof self?self:{},n=(new e.Error).stack;n&&(e._sentryDebugIds=e._sentryDebugIds||{},e._sentryDebugIds[n]="66188821-7793-4ed7-9531-d2ed2a90da2b",e._sentryDebugIdIdentifier="sentry-dbid-66188821-7793-4ed7-9531-d2ed2a90da2b")}catch(e){}}();var _global="undefined"!=typeof window?window:"undefined"!=typeof global?global:"undefined"!=typeof self?self:{};_global.SENTRY_RELEASE={id:"0.9.6"},function(e,n){"object"==typeof exports&&"object"==typeof module?module.exports=n():"function"==typeof define&&define.amd?define([],n):"object"==typeof exports?exports.swh=n():(e.swh=e.swh||{},e.swh.badges=n())}(self,(function(){return function(){"use strict";var e={d:function(n,d){for(var o in d)e.o(d,o)&&!e.o(n,o)&&Object.defineProperty(n,o,{enumerable:!0,get:d[o]})},o:function(e,n){return Object.prototype.hasOwnProperty.call(e,n)},r:function(e){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})}},n={};function d(e,n){var d,o,t=n;if("origin"===e)d=Urls.swh_badge(e,n),o=Urls.browse_origin()+"?origin_url="+n;else{var r=n.indexOf(";");-1!==r?(t=n.slice(0,r),d=Urls.swh_badge_swhid(t),$(".swhid").each((function(e,n){n.id===t&&(o=n.pathname)}))):(d=Urls.swh_badge_swhid(n),o=Urls.browse_swhid(n))}var i=""+window.location.origin+d,a=""+window.location.origin+o,l='\n  <a href="'+a+'">\n    <img class="swh-badge" src="'+i+'" alt="Archived | '+n+'"/>\n  </a>\n  <div>\n    <label>HTML</label>\n    <pre><code class="swh-badge-html html">&lt;a href="'+a+'"&gt;\n    &lt;img src="'+i+'" alt="Archived | '+t+'"/&gt;\n&lt;/a&gt;</code></pre>\n  </div>\n  <div>\n    <label>Markdown</label>\n    <pre><code class="swh-badge-md markdown">[![SWH]('+i+")]("+a+')</code></pre>\n  </div>\n  <div>\n    <label>reStructuredText</label>\n    <pre class="swh-badge-rst">.. image:: '+i+"\n    :target: "+a+"</pre>\n  </div>";swh.webapp.showModalHtml("Software Heritage badge integration",l),swh.webapp.highlightCode(!1,".swh-badge-html"),swh.webapp.highlightCode(!1,".swh-badge-md")}return e.r(n),e.d(n,{showBadgeInfoModal:function(){return d}}),n}()}));
//# sourceMappingURL=badges.8e8737404bb09fc90a9f.js.map